-- Autogenerated list of maps to load and compile:
dgn.load_des_file('des/00init.des')
dgn.load_des_file('des/altar/altar.des')
dgn.load_des_file('des/altar/ashenzari_visionary.des')
dgn.load_des_file('des/altar/ecumenical.des')
dgn.load_des_file('des/altar/kiku_cage.des')
dgn.load_des_file('des/altar/lugonu_bribe.des')
dgn.load_des_file('des/altar/nemelex_the_gamble.des')
dgn.load_des_file('des/altar/okawaru_arena.des')
dgn.load_des_file('des/altar/overflow.des')
dgn.load_des_file('des/altar/trog_burn_book.des')
dgn.load_des_file('des/altar/trog_wizard.des')
dgn.load_des_file('des/altar/vehumet_trees.des')
dgn.load_des_file('des/altar/xom_monty_hall.des')
dgn.load_des_file('des/altar/xom_sheep.des')
dgn.load_des_file('des/altar/yredelemnul_ordeal.des')
dgn.load_des_file('des/arrival/large.des')
dgn.load_des_file('des/arrival/simple.des')
dgn.load_des_file('des/arrival/small.des')
dgn.load_des_file('des/arrival/twisted.des')
dgn.load_des_file('des/branches/abyss.des')
dgn.load_des_file('des/branches/blade.des')
dgn.load_des_file('des/branches/coc.des')
dgn.load_des_file('des/branches/crypt.des')
dgn.load_des_file('des/branches/depths.des')
dgn.load_des_file('des/branches/depths_encompass.des')
dgn.load_des_file('des/branches/dis.des')
dgn.load_des_file('des/branches/elf.des')
dgn.load_des_file('des/branches/geh.des')
dgn.load_des_file('des/branches/geh_lava_maze.des')
dgn.load_des_file('des/branches/hell.des')
dgn.load_des_file('des/branches/hells.des')
dgn.load_des_file('des/branches/hilbert_zone.des')
dgn.load_des_file('des/branches/lair.des')
dgn.load_des_file('des/branches/orc.des')
dgn.load_des_file('des/branches/pan.des')
dgn.load_des_file('des/branches/shoals.des')
dgn.load_des_file('des/branches/slime.des')
dgn.load_des_file('des/branches/snake.des')
dgn.load_des_file('des/branches/spider.des')
dgn.load_des_file('des/branches/spider_jumping.des')
dgn.load_des_file('des/branches/swamp.des')
dgn.load_des_file('des/branches/tar.des')
dgn.load_des_file('des/branches/temple.des')
dgn.load_des_file('des/branches/temple_compat.des')
dgn.load_des_file('des/branches/tomb.des')
dgn.load_des_file('des/branches/vaults.des')
dgn.load_des_file('des/branches/vaults_rooms_empty.des')
dgn.load_des_file('des/branches/vaults_rooms_hard.des')
dgn.load_des_file('des/branches/vaults_rooms_standard.des')
dgn.load_des_file('des/branches/zot.des')
dgn.load_des_file('des/builder/alphashops.des')
dgn.load_des_file('des/builder/arena.des')
dgn.load_des_file('des/builder/custombooks.des')
dgn.load_des_file('des/builder/decor.des')
dgn.load_des_file('des/builder/dummy.des')
dgn.load_des_file('des/builder/layout.des')
dgn.load_des_file('des/builder/layout_caves.des')
dgn.load_des_file('des/builder/layout_cc.des')
dgn.load_des_file('des/builder/layout_cellular.des')
dgn.load_des_file('des/builder/layout_city.des')
dgn.load_des_file('des/builder/layout_delve.des')
dgn.load_des_file('des/builder/layout_forest.des')
dgn.load_des_file('des/builder/layout_geoelf.des')
dgn.load_des_file('des/builder/layout_geoelf_castle.des')
dgn.load_des_file('des/builder/layout_grids.des')
dgn.load_des_file('des/builder/layout_halls.des')
dgn.load_des_file('des/builder/layout_loops.des')
dgn.load_des_file('des/builder/layout_overlapping_boxes.des')
dgn.load_des_file('des/builder/layout_pan_divisions.des')
dgn.load_des_file('des/builder/layout_pools.des')
dgn.load_des_file('des/builder/layout_vaults.des')
dgn.load_des_file('des/builder/rooms.des')
dgn.load_des_file('des/builder/shops.des')
dgn.load_des_file('des/builder/uniques.des')
dgn.load_des_file('des/portals/arena.des')
dgn.load_des_file('des/portals/bailey.des')
dgn.load_des_file('des/portals/bazaar.des')
dgn.load_des_file('des/portals/crucible.des')
dgn.load_des_file('des/portals/desolation.des')
dgn.load_des_file('des/portals/gauntlet.des')
dgn.load_des_file('des/portals/icecave.des')
dgn.load_des_file('des/portals/necropolis.des')
dgn.load_des_file('des/portals/ossuary.des')
dgn.load_des_file('des/portals/sewer.des')
dgn.load_des_file('des/portals/trove.des')
dgn.load_des_file('des/portals/volcano.des')
dgn.load_des_file('des/portals/wizlab.des')
dgn.load_des_file('des/portals/ziggurat.des')
dgn.load_des_file('des/serial/aquarium.des')
dgn.load_des_file('des/serial/bayou.des')
dgn.load_des_file('des/serial/column_ruins.des')
dgn.load_des_file('des/serial/corridors.des')
dgn.load_des_file('des/serial/curves.des')
dgn.load_des_file('des/serial/demonic.des')
dgn.load_des_file('des/serial/forest.des')
dgn.load_des_file('des/serial/glass.des')
dgn.load_des_file('des/serial/gnoll_camp.des')
dgn.load_des_file('des/serial/ice.des')
dgn.load_des_file('des/serial/library.des')
dgn.load_des_file('des/serial/magic_research.des')
dgn.load_des_file('des/serial/park.des')
dgn.load_des_file('des/serial/rivers.des')
dgn.load_des_file('des/serial/sigils.des')
dgn.load_des_file('des/serial/subway.des')
dgn.load_des_file('des/serial/undead.des')
dgn.load_des_file('des/serial/window.des')
dgn.load_des_file('des/sprint/arena_sprint.des')
dgn.load_des_file('des/sprint/fedhas.des')
dgn.load_des_file('des/sprint/linesprint.des')
dgn.load_des_file('des/sprint/meat.des')
dgn.load_des_file('des/sprint/menkaure.des')
dgn.load_des_file('des/sprint/pitsprint.des')
dgn.load_des_file('des/sprint/red_sonja.des')
dgn.load_des_file('des/sprint/sprint_mu.des')
dgn.load_des_file('des/sprint/zigsprint.des')
dgn.load_des_file('des/test.des')
dgn.load_des_file('des/test/suite-los.des')
dgn.load_des_file('des/test/suite.des')
dgn.load_des_file('des/traps/monster_drop.des')
dgn.load_des_file('des/traps/rats_trap.des')
dgn.load_des_file('des/tutorial/lesson1.des')
dgn.load_des_file('des/tutorial/lesson2.des')
dgn.load_des_file('des/tutorial/lesson3.des')
dgn.load_des_file('des/tutorial/lesson4.des')
dgn.load_des_file('des/tutorial/lesson5.des')
dgn.load_des_file('des/variable/ambush.des')
dgn.load_des_file('des/variable/arcadia.des')
dgn.load_des_file('des/variable/compat.des')
dgn.load_des_file('des/variable/d_encompass.des')
dgn.load_des_file('des/variable/float.des')
dgn.load_des_file('des/variable/geyser.des')
dgn.load_des_file('des/variable/grated_community.des')
dgn.load_des_file('des/variable/large_abstract.des')
dgn.load_des_file('des/variable/large_themed.des')
dgn.load_des_file('des/variable/lemuel_castle.des')
dgn.load_des_file('des/variable/mini.des')
dgn.load_des_file('des/variable/mini_features.des')
dgn.load_des_file('des/variable/mini_monsters.des')
dgn.load_des_file('des/variable/polyominoes.des')
dgn.load_des_file('des/variable/the_bubble.des')
dgn.load_des_file('des/variable/the_grid.des')
dgn.load_des_file('des/variable/wad_woods.des')
dgn.load_des_file('des/variable/winding_woodlands.des')
